@@ -10,9 +10,9 @@ class CoreTest(TestCaseHelper, TestCase):
 
     fixture_src = __file__
 
-    def setUp(self):
+    def setUp(self, run_svcs=False):
         super().setUp()
-        self.meta = MetaService()
+        self.meta = MetaService(run_svcs=run_svcs)
         self.smast = self.meta.build_super_master()
         self.mast = self.meta.build_master(h=self.smast._h)
 
